@@ -6,6 +6,7 @@ from parse_it.file.toml import *
 from parse_it.file.hcl import *
 from parse_it.file.yaml import *
 from parse_it.file.json import *
+from parse_it.file.env import *
 from parse_it.file.ini import *
 from parse_it.file.xml import *
 from parse_it.file.file_reader import *
@@ -197,6 +198,19 @@ class BaseTests(TestCase):
         }
         self.assertDictEqual(reply, expected_reply)
 
+    def test_env_file(self):
+        reply = parse_env_file(test_files_location + "/test.env")
+        expected_reply = {
+            'file_type': 'env',
+            'test_string': 'testing',
+            'test_bool_true': 'true',
+            'test_bool_false': 'false',
+            'test_int': '123',
+            'test_float': '123.123',
+            'test_list': '["test1","test2","test3"]',
+            'test_env': '{"test_env_key": "test_env_value"}'}
+        self.assertDictEqual(reply, expected_reply)
+
     def test_hcl(self):
         reply = parse_hcl_file(test_files_location + "/test.hcl")
         expected_reply = {
@@ -348,6 +362,9 @@ class BaseTests(TestCase):
 
     def test_parser_init_no_recurse(self):
         expected_config_files_dict = {
+            'env': [
+                "test.env"
+            ],
             'json': [
                 'test.json'
             ],
@@ -368,8 +385,8 @@ class BaseTests(TestCase):
                 'test.xml'
             ]
         }
-        expected_config_type_priority = ['cli_args', 'env_vars', 'json', 'yaml', 'yml', 'toml', 'tml', 'hcl', 'tf',
-                                         'conf', 'cfg', 'ini', 'xml']
+        expected_config_type_priority = ['cli_args', 'env_vars', "env", 'json', 'yaml', 'yml', 'toml', 'tml', 'hcl',
+                                         'tf', 'conf', 'cfg', 'ini', 'xml']
         parser = ParseIt(config_type_priority=None, global_default_value=None, type_estimate=True,
                          force_envvars_uppercase=True, config_location=test_files_location, envvar_prefix=None)
         self.assertDictEqual(parser.config_files_dict, expected_config_files_dict)
@@ -382,6 +399,9 @@ class BaseTests(TestCase):
 
     def test_parser_init_recurse(self):
         expected_config_files_dict = {
+            'env': [
+                'test.env'
+            ],
             'json': [
                 'test.json',
                 'test_subfolder_1/test_sub_subfolder_2/test_subfolder_2.json',
@@ -405,8 +425,8 @@ class BaseTests(TestCase):
                 'test.xml'
             ]
         }
-        expected_config_type_priority = ['cli_args', 'env_vars', 'json', 'yaml', 'yml', 'toml', 'tml', 'hcl', 'tf',
-                                         'conf', 'cfg', 'ini', 'xml']
+        expected_config_type_priority = ['cli_args', 'env_vars', 'env', 'json', 'yaml', 'yml', 'toml', 'tml', 'hcl',
+                                         'tf', 'conf', 'cfg', 'ini', 'xml']
         parser = ParseIt(config_type_priority=None, global_default_value=None, type_estimate=True, recurse=True,
                          force_envvars_uppercase=True, config_location=test_files_location, envvar_prefix=None)
         self.assertDictEqual(parser.config_files_dict, expected_config_files_dict)
@@ -419,6 +439,7 @@ class BaseTests(TestCase):
 
     def test_parser_init_single_file(self):
         expected_config_files_dict = {
+            'env': [],
             'cfg': [],
             'conf': [],
             'hcl': [],
@@ -433,8 +454,8 @@ class BaseTests(TestCase):
             'yaml': [],
             'yml': []
         }
-        expected_config_type_priority = ['cli_args', 'env_vars', 'json', 'yaml', 'yml', 'toml', 'tml', 'hcl', 'tf',
-                                         'conf', 'cfg', 'ini', 'xml']
+        expected_config_type_priority = ['cli_args', 'env_vars', 'env', 'json', 'yaml', 'yml', 'toml', 'tml', 'hcl',
+                                         'tf', 'conf', 'cfg', 'ini', 'xml']
         parser = ParseIt(config_type_priority=None, global_default_value=None, type_estimate=True,
                          force_envvars_uppercase=True, config_location=test_files_location + "/test.json",
                          envvar_prefix=None)
@@ -446,6 +467,16 @@ class BaseTests(TestCase):
         self.assertIsNone(parser.global_default_value)
         self.assertTrue(parser.type_estimate)
 
+    def test_parser_init_envvar_divider_none(self):
+        parser = ParseIt(config_location=test_files_location + "/test.json", envvar_divider=None)
+        self.assertFalse(parser.nest_envvars)
+
+    def test_parser_init_envvar_divider_set(self):
+        test_envvar_divider = "_"
+        parser = ParseIt(config_location=test_files_location + "/test.json", envvar_divider=test_envvar_divider)
+        self.assertTrue(parser.nest_envvars)
+        self.assertEqual(test_envvar_divider, parser.envvar_divider)
+
     def test_parser_custom_suffix_mapping_set_config_type_priority_not_set_raise_warning(self):
         with self.assertWarns(Warning):
             ParseIt(config_location=test_files_location, custom_suffix_mapping={"yaml": ["custom"]})
@@ -454,9 +485,12 @@ class BaseTests(TestCase):
         parser = ParseIt(config_location=test_files_location, custom_suffix_mapping={"yaml": ["custom"]},
                          config_type_priority=["custom"] + VALID_FILE_TYPE_EXTENSIONS)
         expected_config_type_priority = ["custom"] + VALID_FILE_TYPE_EXTENSIONS
-        expected_valid_type_extension = ['json', 'yaml', 'yml', 'toml', 'tml', 'hcl', 'tf', 'conf', 'cfg', 'ini', 'xml',
-                                         'custom']
+        expected_valid_type_extension = ['env', 'json', 'yaml', 'yml', 'toml', 'tml', 'hcl', 'tf', 'conf', 'cfg', 'ini',
+                                         'xml', 'custom']
         expected_suffix_file_type_mapping = {
+            'env': [
+                'env'
+            ],
             'json': [
                 'json'
             ],
@@ -493,7 +527,7 @@ class BaseTests(TestCase):
     def test_parser_read_configuration_variable(self):
         parser = ParseIt(config_location=test_files_location)
         reply_json = parser.read_configuration_variable("file_type")
-        self.assertEqual(reply_json, "json")
+        self.assertEqual(reply_json, "env")
         reply_yaml = parser.read_configuration_variable("test_yaml")
         self.assertDictEqual(reply_yaml, {'test_yaml_key': 'test_yaml_value'})
 
@@ -506,6 +540,9 @@ class BaseTests(TestCase):
         parser = ParseIt(config_location=test_files_location, recurse=False)
         reply = parser.read_configuration_variable("test_json_subfolder")
         expected_config_files_dict = {
+            'env': [
+                'test.env'
+            ],
             'json': [
                 'test.json'
             ],
@@ -555,7 +592,7 @@ class BaseTests(TestCase):
     def test_parser_read_configuration_variable_required_true_value_given_file(self):
         parser = ParseIt(config_location=test_files_location)
         reply_json = parser.read_configuration_variable("file_type", required=True)
-        self.assertEqual(reply_json, "json")
+        self.assertEqual(reply_json, "env")
 
     def test_parser_read_configuration_variable_required_true_value_given_envvar(self):
         os.environ["TEST_ENVVAR_ESTIMATE_TRUE_INT"] = "123"
@@ -570,7 +607,7 @@ class BaseTests(TestCase):
 
     def test_parser_read_configuration_variable_config_type_priority(self):
         os.environ["TEST_ENVVAR_ESTIMATE_TRUE_INT"] = "123"
-        parser = ParseIt(config_type_priority=["yaml", "toml", "ini", "json", "envvars"],
+        parser = ParseIt(config_type_priority=["yaml", "toml", "ini", "json", "envvars", "env"],
                          config_location=test_files_location)
         reply = parser.read_configuration_variable("file_type")
         self.assertEqual(reply, "yaml")
@@ -582,6 +619,8 @@ class BaseTests(TestCase):
         self.assertDictEqual(reply, {'test_json_key': 'test_json_value'})
         reply = parser.read_configuration_variable("TEST_ENVVAR_ESTIMATE_TRUE_INT")
         self.assertEqual(reply, 123)
+        reply = parser.read_configuration_variable("test_list")
+        self.assertEqual(reply, ['test1', 'test2', 'test3'])
 
     def test_parser_read_configuration_variable_type_estimate_false(self):
         parser = ParseIt(type_estimate=False, config_location=test_files_location)
@@ -672,7 +711,7 @@ class BaseTests(TestCase):
     def test_parser_read_configuration_variable_config_location(self):
         parser = ParseIt(config_location=test_files_location)
         reply_json = parser.read_configuration_variable("file_type")
-        self.assertEqual(reply_json, "json")
+        self.assertEqual(reply_json, "env")
         reply_json = parser.read_configuration_variable("test_float")
         self.assertEqual(reply_json, 123.123)
         reply_yaml = parser.read_configuration_variable("test_yaml")
@@ -711,6 +750,12 @@ class BaseTests(TestCase):
         reply = parser.read_configuration_variable("test_envvar_estimate_true_int")
         self.assertEqual(reply, 123)
 
+    def test_parser_read_configuration_variable_envvar_nested(self):
+        parser = ParseIt(config_location=test_files_location, envvar_divider="_")
+        os.environ["TEST_ENVVAR_NESTED"] = "123"
+        reply = parser.read_configuration_variable("test_envvar_nested")
+        self.assertDictEqual(reply, {"test": {"envvar": {"nested": 123}}})
+
     def test_envvar_defined_true(self):
         os.environ["TEST_ENV"] = "123"
         reply = envvar_defined("TEST_ENV")
@@ -746,6 +791,63 @@ class BaseTests(TestCase):
             self.assertEqual(reply["TEST_ENV"], "123")
             self.assertEqual(reply["test_env_lowercase"], "456")
 
+    def test_envvars_split_force_uppercase_false(self):
+        expected_reply = {'dict': {'subdict': {'key': 'example_value'}}}
+        test_key = "dict_subdict_key"
+        test_value = "example_value"
+        reply = split_envvar(test_key, test_value, divider="_")
+        self.assertEqual(expected_reply, reply)
+
+    def test_envvars_split_force_uppercase_true(self):
+        expected_reply = {'dict': {'subdict': {'key': 'example_value'}}}
+        test_key = "dict_subdict_key"
+        test_value = "example_value"
+        reply = split_envvar(test_key, test_value, divider="_")
+        self.assertEqual(expected_reply, reply)
+
+    def test_envvars_split_no_nesting_needed(self):
+        expected_reply = {'dict': 'example_value'}
+        test_key = "dict"
+        test_value = "example_value"
+        reply = split_envvar(test_key, test_value, divider="_")
+        self.assertEqual(expected_reply, reply)
+
+    def test_envvars_split_custom_divider(self):
+        expected_reply = {'dict': {'subdict': {'key': 'example_value'}}}
+        test_key = "dict.subdict.key"
+        test_value = "example_value"
+        reply = split_envvar(test_key, test_value, divider=".")
+        self.assertEqual(expected_reply, reply)
+
+    def test_envvars_split_envvar_combained_dict(self):
+        test_envvars = {"TEST_SPLIT_UPPERCASE1": "test123", "test_split_lowercase1": "test456"}
+        with mock.patch.dict(os.environ, test_envvars):
+            reply = split_envvar_combained_dict()
+            self.assertEqual(type(reply), dict)
+            self.assertEqual(reply["test"]["split"]["uppercase1"], "test123")
+            self.assertEqual(reply["test"]["split"]["lowercase1"], "test456")
+
+    def test_envvars_split_envvar_combained_dict_custom_divider(self):
+        test_envvars = {"TEST.SPLIT.UPPER_CASE2": "test123", "test.split.lower_case2": "test456"}
+        with mock.patch.dict(os.environ, test_envvars):
+            reply = split_envvar_combained_dict(divider=".")
+            self.assertEqual(type(reply), dict)
+            self.assertEqual(reply["test"]["split"]["upper_case2"], "test123")
+            self.assertEqual(reply["test"]["split"]["lower_case2"], "test456")
+
+    def test_envvars_split_envvar_combained_dict_force_uppercase_false(self):
+        test_envvars = {
+            "TEST_SPLIT_UPPERCASE3": "test123",
+            "test_split_lowercase3": "test456",
+            "test_split_lowercase4": "test789"
+        }
+        with mock.patch.dict(os.environ, test_envvars):
+            reply = split_envvar_combained_dict(force_uppercase=False)
+            self.assertEqual(type(reply), dict)
+            self.assertEqual(reply["TEST"]["SPLIT"]["UPPERCASE3"], "test123")
+            self.assertEqual(reply["test"]["split"]["lowercase3"], "test456")
+            self.assertEqual(reply["test"]["split"]["lowercase4"], "test789")
+
     def test_parser_config_found_in_key(self):
         parser = ParseIt(config_location=test_files_location)
         config_found_reply, config_value_reply = parser._check_config_in_dict("test_key", {"test_key": "test_value"})
@@ -985,7 +1087,7 @@ class BaseTests(TestCase):
     def test_parser_read_configuration_variable_check_allowed_types_None(self):
         parser = ParseIt(config_location=test_files_location)
         reply_json = parser.read_configuration_variable("file_type", allowed_types=None)
-        self.assertEqual(reply_json, "json")
+        self.assertEqual(reply_json, "env")
 
     def test_parser_read_configuration_variable_check_allowed_not_in_types_list(self):
         with self.assertRaises(TypeError):
@@ -995,11 +1097,11 @@ class BaseTests(TestCase):
     def test_parser_read_configuration_variable_check_allowed_in_types_list(self):
         parser = ParseIt(config_location=test_files_location)
         reply_json = parser.read_configuration_variable("file_type", allowed_types=[int, str, dict, None, float])
-        self.assertEqual(reply_json, "json")
+        self.assertEqual(reply_json, "env")
 
     def test_parser_read_multiple_configuration_variables_works_multiple_list(self):
         expected_reply = {
-            'file_type': 'json',
+            'file_type': 'env',
             'test_hcl': {
                 'test_hcl_name': {
                     'test_hcl_key': 'test_hcl_value'
@@ -1018,7 +1120,7 @@ class BaseTests(TestCase):
 
     def test_parser_read_multiple_configuration_variables_default_value(self):
         expected_reply = {
-            'file_type': 'json',
+            'file_type': 'env',
             'non_existing_value': 'test'
         }
         parser = ParseIt(config_location=test_files_location)
@@ -1039,7 +1141,7 @@ class BaseTests(TestCase):
     def test_parser_read_all_configuration_variables(self):
         parser = ParseIt(config_location=test_files_location)
         reply = parser.read_all_configuration_variables()
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1048,11 +1150,26 @@ class BaseTests(TestCase):
         self.assertDictEqual(reply["test_dict"], {'hcl_dict_key': 'hcl_dict_value'})
         self.assertListEqual(reply["test_list"], ['test1', 'test2', 'test3'])
 
+    def test_parser_read_all_configuration_variables_nest_envvars(self):
+        parser = ParseIt(config_location=test_files_location, envvar_divider="_", force_envvars_uppercase=True)
+        test_envvars = {"test_split_nesting1": "test123", "test_split_nesting2": "test456"}
+        with mock.patch.dict(os.environ, test_envvars):
+            reply = parser.read_all_configuration_variables()
+            self.assertDictEqual(reply["test"]["split"], {'nesting1': 'test123', 'nesting2': 'test456'})
+            self.assertEqual(reply["file_type"], "env")
+            self.assertEqual(reply["test_string"], "testing")
+            self.assertTrue(reply["test_bool_true"])
+            self.assertFalse(reply["test_bool_false"])
+            self.assertEqual(reply["test_int"], 123)
+            self.assertEqual(reply["test_float"], 123.123)
+            self.assertDictEqual(reply["test_dict"], {'hcl_dict_key': 'hcl_dict_value'})
+            self.assertListEqual(reply["test_list"], ['test1', 'test2', 'test3'])
+
     def test_parser_read_all_configuration_variables_default_value_given(self):
         parser = ParseIt(config_location=test_files_location)
         reply = parser.read_all_configuration_variables(default_value={"default_value_test": "it_works"})
         self.assertEqual(reply["default_value_test"], "it_works")
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1064,7 +1181,7 @@ class BaseTests(TestCase):
     def test_parser_read_all_configuration_variables_none_default_value_given(self):
         parser = ParseIt(config_location=test_files_location)
         reply = parser.read_all_configuration_variables(default_value=None)
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1076,7 +1193,7 @@ class BaseTests(TestCase):
     def test_parser_read_all_configuration_variables_empty_default_value_given(self):
         parser = ParseIt(config_location=test_files_location)
         reply = parser.read_all_configuration_variables(default_value={})
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1088,7 +1205,7 @@ class BaseTests(TestCase):
     def test_parser_read_all_configuration_variables_required_given(self):
         parser = ParseIt(config_location=test_files_location)
         reply = parser.read_all_configuration_variables(required=["file_type", "test_int"])
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1100,7 +1217,7 @@ class BaseTests(TestCase):
     def test_parser_read_all_configuration_variables_empty_required_given(self):
         parser = ParseIt(config_location=test_files_location)
         reply = parser.read_all_configuration_variables(required=[])
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1112,7 +1229,7 @@ class BaseTests(TestCase):
     def test_parser_read_all_configuration_variables_none_required_given(self):
         parser = ParseIt(config_location=test_files_location)
         reply = parser.read_all_configuration_variables(required=None)
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1133,7 +1250,7 @@ class BaseTests(TestCase):
                 str
             ]
         })
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1145,7 +1262,7 @@ class BaseTests(TestCase):
     def test_parser_read_all_configuration_variables_empty_allowed_types_given(self):
         parser = ParseIt(config_location=test_files_location)
         reply = parser.read_all_configuration_variables(allowed_types={})
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1157,7 +1274,7 @@ class BaseTests(TestCase):
     def test_parser_read_all_configuration_variables_none_allowed_types_given(self):
         parser = ParseIt(config_location=test_files_location)
         reply = parser.read_all_configuration_variables(allowed_types=None)
-        self.assertEqual(reply["file_type"], "json")
+        self.assertEqual(reply["file_type"], "env")
         self.assertEqual(reply["test_string"], "testing")
         self.assertTrue(reply["test_bool_true"])
         self.assertFalse(reply["test_bool_false"])
@@ -1201,7 +1318,7 @@ class BaseTests(TestCase):
             parser = ParseIt(config_location=test_files_location, type_estimate=True)
             reply = parser.read_all_configuration_variables()
             self.assertEqual(reply["test_env_type_estimate"], 123)
-            self.assertEqual(reply["file_type"], "json")
+            self.assertEqual(reply["file_type"], "env")
             self.assertEqual(reply["test_string"], "testing")
             self.assertTrue(reply["test_bool_true"])
             self.assertFalse(reply["test_bool_false"])
@@ -1216,7 +1333,7 @@ class BaseTests(TestCase):
             parser = ParseIt(config_location=test_files_location, type_estimate=True)
             reply = parser.read_all_configuration_variables()
             self.assertEqual(reply["test_cli_key_no_folder"], "test_value")
-            self.assertEqual(reply["file_type"], "json")
+            self.assertEqual(reply["file_type"], "env")
             self.assertEqual(reply["test_string"], "testing")
             self.assertTrue(reply["test_bool_true"])
             self.assertFalse(reply["test_bool_false"])
@@ -1236,3 +1353,43 @@ class BaseTests(TestCase):
         self.assertEqual(reply["test_float"], 123.123)
         self.assertDictEqual(reply["test_dict"], {'hcl_dict_key': 'hcl_dict_value'})
         self.assertListEqual(reply["test_list"], ['test1', 'test2', 'test3'])
+
+    def test_parser_read_all_configuration_variables_recursive_false(self):
+        parser = ParseIt(config_location=test_files_location, recurse=False)
+        reply = parser.read_all_configuration_variables()
+        self.assertEqual(reply["file_type"], "env")
+        self.assertEqual(reply["test_string"], "testing")
+        self.assertTrue(reply["test_bool_true"])
+        self.assertFalse(reply["test_bool_false"])
+        self.assertEqual(reply["test_int"], 123)
+        self.assertEqual(reply["test_float"], 123.123)
+        self.assertDictEqual(reply["test_dict"], {'hcl_dict_key': 'hcl_dict_value'})
+        self.assertListEqual(reply["test_list"], ['test1', 'test2', 'test3'])
+
+    def test_parser_read_all_configuration_variables_recursive_true(self):
+        parser = ParseIt(config_location=test_files_location, recurse=True)
+        reply = parser.read_all_configuration_variables()
+        self.assertEqual(reply["file_type"], "env")
+        self.assertEqual(reply["test_string"], "testing")
+        self.assertTrue(reply["test_bool_true"])
+        self.assertTrue(reply["test_json_subfolder"])
+        self.assertFalse(reply["test_bool_false"])
+        self.assertEqual(reply["test_int"], 123)
+        self.assertEqual(reply["test_float"], 123.123)
+        self.assertDictEqual(reply["test_dict"], {'hcl_dict_key': 'hcl_dict_value'})
+        self.assertListEqual(reply["test_list"], ['test1', 'test2', 'test3'])
+
+    def test_parser_read_all_configuration_variables_check_order(self):
+        parser = ParseIt(config_location=test_files_location, config_type_priority=["env_vars", "hcl", "json"])
+        test_envvars = {"TEST_INT": "12345"}
+        with mock.patch.dict(os.environ, test_envvars):
+            reply = parser.read_all_configuration_variables()
+            self.assertEqual(reply["file_type"], "hcl")
+            self.assertEqual(reply["test_string"], "testing")
+            self.assertTrue(reply["test_bool_true"])
+            self.assertFalse(reply["test_bool_false"])
+            self.assertEqual(reply["test_int"], 12345)
+            self.assertEqual(reply["test_float"], 123.123)
+            self.assertDictEqual(reply["test_dict"], {'hcl_dict_key': 'hcl_dict_value'})
+            self.assertListEqual(reply["test_list"], ['test1', 'test2', 'test3'])
+            self.assertDictEqual(reply["test_json"], {"test_json_key": "test_json_value"})
